/*  kgldisk.cpp - 
    This file is part of KGL
    Copyright (C) 2000,2001  Antonio Larrosa Jimenez
    KGL' homepage : http://devel-home.kde.org/~larrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "kgldisk.h"
#include <math.h>
#include <stdio.h>

KGLDisk::KGLDisk(GLfloat x1, GLfloat y1, GLfloat z1,
    GLfloat nx, GLfloat ny, GLfloat nz, GLfloat r, int slices,
    QImage texture) : KGLObject(x1,y1,z1)
{
  GLfloat h=sqrt(nx*nx+ny*ny+nz*nz);

  /* Let's calculate the angle between (0,0,0)(0,0,h) and (0,0,0)(x2-x1,y2-y1,z2-z1)
   * We need first to get the orthogonal projection of (0,0,h) onto the line 
   * generated by the vector (x2-x1,y2-y1,z2-z1) 
   *
   * We have to get a point (x,y,z) which verifies:
   *  x-x1   y-y2    z-z2
   *  ---- = ----  = ----
   *  x2-x1  y2-y1   z2-z1
   *
   * and
   *
   * (a-x)*(x1-x)+(b-y)*(y1-y)+(c-z)*(z1-z)=0
   *
   * That is, (x,y,z) which verifies :
   *
   *    x      y
   *  ---- = ----
   *  x2-x1  y2-y1
   *
   *    y       z
   *  ----  = ----
   *  y2-y1   z2-z1
   *
   * and
   *
   *  2   2
   * x  +y  +z*(z-h)=0
   *
   * Therefore, we must have:
   *
   *             (z2-z1)*h*(y2-y1)
   * y =  -------------------------------------
   *                2           2           2
   *         (x2-x1)   + (y2-y1)   + (z2-z1)
   *
   * and:
   *      y*(x2-x1)       y*(z2-z1)
   *  x = ---------   z = ---------
   *        y2-y1           y2-y1
   */

  GLfloat dp=h*nz/(nx*nx+ny*ny+nz*nz);
  GLfloat px=nx*dp;
  GLfloat py=ny*dp;
  GLfloat pz=nz*dp;                                                               
  GLfloat dpx=sqrt(px*px+py*py+pz*pz);
  GLfloat dpy=sqrt(px*px+py*py+(h-pz)*(h-pz));
  // rho is the angle between (0,0,h) and (dx,dy,dz)
  // GLfloat rho=(dpy==0) ? 0 : atan2(dpy,dpx);
  GLfloat rho=atan2(dpy,dpx);
  if (dp<0.0) rho=M_PI-rho;

  // v is the vectorial product of (0,0,h) and d
  GLfloat vx=-h*ny;
  GLfloat vy=h*nx;
  GLfloat vz=0;

  m_list = glGenLists(1);
  glNewList(m_list, GL_COMPILE);
  GLUquadricObj* q = gluNewQuadric();
  gluQuadricDrawStyle( q, (GLenum) GLU_FILL );
  gluQuadricNormals( q, (GLenum) GLU_SMOOTH );
  glEnable(GL_NORMALIZE);

  glPushMatrix();
  glMatrixMode(GL_MODELVIEW);

  glRotatef(rho*360.0/(2*M_PI),vx,vy,vz);
  
  if (!texture.isNull()) 
  {
    gluQuadricTexture(q, GL_TRUE);
    texture = texture.convertDepth( 32 ).mirror().swapRGB(); 
    glTexImage2D( GL_TEXTURE_2D, 0, 3, texture.width(), texture.height(), 0,
	GL_RGBA, GL_UNSIGNED_BYTE, texture.bits() );
  }
  gluDisk(q, 0.0, r, slices, 1);

  gluQuadricOrientation(q, (GLenum)GLU_INSIDE);
  gluDisk(q, 0.0, r, slices, 1);
  gluQuadricOrientation(q, (GLenum)GLU_OUTSIDE);
  glPopMatrix();
  glEndList();

  gluDeleteQuadric(q);


}

